@@ -173,8 +173,9 @@ class NStep(BaseTracer):
             gamman = self._gamman
         else:
             # no more bootstrapping
-            obs_next, a_next, v_next, pi_next, done = obs, a, v, pi, True
-            gamman = self._gammas[len(rs)]
+            v_next = 0
+            done = True
+            gamman = self._gammas[len(rs) - 1]
         
         Rn += v_next * gamman
 
@@ -222,8 +223,9 @@ class PNStep(NStep):
             gamman = self._gamman
         else:
             # no more bootstrapping
-            obs_next, a_next, v_next, pi_next, done = obs, a, v, pi, True
-            gamman = self._gammas[len(rs)]
+            v_next = 0
+            done = True
+            gamman = self._gammas[len(rs) - 1]
         
         Rn += v_next * gamman
 
